const join = require('path').join
const loaderUtils = require('loader-utils')
const loadLayout = require('./content/loadLayout')
const cheerio = require('cheerio')
const markdownit = require('markdown-it')
const fm = require('front-matter')

module.exports = function (source) {
  this.cacheable()

  const options = loaderUtils.getOptions(this)
  const config = options.config
  const filePath = this.resourcePath
  const pageData = config.contentData[filePath].page


  const $layout = cheerio.load(loadLayout(filePath, config, this), {
    xmlMode: true,                // special tags
    decodeEntities: false,        // allow HTML entities (needed for preprocessors)
    lowerCaseAttributeNames: false,
    lowerCaseTags: false
  })

  return `
    ${renderScript(source, pageData)}
    ${$layout.html('template')}
    ${$layout.html('style')}
  `
}

/**
 * Creates script element for the file's metadata.
 */
function renderScript(source, pageData) {
  const parser = markdownit({
    preset: 'default',
    html: true
    // highlight: renderHighlight
  })

  const markdown = fm(source).body
  const data = Object.assign({}, pageData, {
    content: parser.render(markdown) // md -> html
  })

  return `
    <script>
    export default {
      data() {
        return ${JSON.stringify(data)}
      }
    }
    </script>
  `
}
